use crate::utils::parsing;

use super::{item_mapping::ItemMapping, UnsignedNumber};

pub fn parse(input: &str) -> (Vec<UnsignedNumber>, Vec<Vec<ItemMapping>>) {
    let mut blocks = parsing::blocks_of_lines_seperated_by_empty_lines(input);
    let seeds = {
        let first_block = blocks.next().unwrap();
        let first_line = first_block.first().unwrap();
        first_line
            .strip_prefix("seeds:")
            .unwrap()
            .split_whitespace()
            .map(|not_parsed| not_parsed.parse::<UnsignedNumber>().unwrap())
            .collect()
    };

    let mappings: Vec<Vec<ItemMapping>> = blocks
        .map(|lines| {
            lines
                .into_iter()
                .skip(1)
                .map(|to_parse| to_parse.parse::<ItemMapping>().unwrap())
                .collect()
        })
        .collect();
    (seeds, mappings)
}

#[cfg(test)]
mod testing {
    #[test]
    fn should_parse() {
        let input = include_str!("day_example_input.txt");
        let actual = super::parse(input);
        insta::assert_debug_snapshot!(actual);
    }
}
